/**********************************************************************\
**
**                         Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Api.h
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Xpm_Api (Application Programming Interface)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Application programming interface of xpm printer driver
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
**               R2-V1.01a / 2009NOV24 / HOR / M-R2-001-HOR
**                  Configuration data extended with test page on printer close feature.  
**               R2-V1_01a / 2009DEC04 / HOR / M-R2-006-HOR 
**                  Extension of configuration data with multi level dot history setting.
**               R2-V1.01a / 2010JAN26 / HOR / M-R2-007-HOR
**                  Configuration data extended with option "disable eject on paper-end"
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR01 / HOR / M-R2-009-HOR
**                  Configuration data extended with initialization delay setting.
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2010MAY12 / HOR / M-R2-012-HOR
**                  Configuration data extended with paper-pre-end pass-trhough flags.
**               R2-V1.01c / 2010FEB09 / HOR / M-R2-017-HOR
**                  Printer settings extended with option pre-pull form before print, which assures 
**                  that the form is pulled from the paper roll before printing.
**               R2-V1.01c / 2010FEB09 / HOR / M-R2-018-HOR
**                  Command GS a+[n1]+[n2] extended with multi barcode inquiry request ([n1]=0x01).
**               R2-V1.01c / 2011APR20 / HOR / M-R2-019-HOR
**                  Command GS a+[n1]+[n2] extended with ticket information request ([n1]=0x02).
**                  Structure TS_XPM_API_PCKT_PRINTER_TICKET added
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / M-R2-020-HOR
**                   Text GB3212 changed to GB2312.
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2012DEC05 / HOR / M-R2-028-HOR
**                  Application programming interface extended with two new functions Xpm_Api_Pio_PrintImage(..) and 
**                  Xpm_Api_Pio_Print_Image_Ex(..), which both render a 24 bit image to a monochrome bitmap and print it.
**               R2-V1.01e / 2012DEC05 / HOR / M-R2-029-HOR
**                  Application programming interface extended with two new functions Xpm_-Api_Pio_RenderImage(..) 
**                  and Xpm_Api_Pio_RenderImageEx (..), which both render a 24 bit image to a monochrome bitmap 
**                  and stores it in a file.
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2015JAN26 / HOR / M-R2-037-HOR
**                  Configuration data extended with paper path option "mark cut offset is relative to mark center"
**               R2-V1.01f / 2015FEB15 / HOR / M-R2-039-HOR
**                  Failure regarding requesting and processing of life data resolved (life data structure in API was not BYTE aligned).
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01g / 2016MAY24 / HOR / M-R2-045-HOR
**                  Configuration data extended with a mark threshold correction, which allows reduction of the paper mark detection threshold
**                  (up to 0.5 Volt in 0.1 Volt steps). The paper mark threshold correction is only used when a reflex sensor is used for paper mark detection.
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                  Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                  Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06a / 2020MAR04 / HOR / M-R7-003-HOR
**                  Configuration data extended with a new feature which makes it possible to trigger (test) printouts when a 
**                  specified sensor switches from low to high or vice versa.
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

#ifndef __XPM_API_H 
#define __XPM_API_H 

/*
** COMPILER
*/

/*
** INCLUDES
*/

/*
** MACROS 
*/

//-- XPM printer access options
#define D_XPM_API_PIO_OPT_NONE                                                0x00000000 /* Option indicating NO OPTIONS                      */
#define D_XPM_API_PIO_OPT_NO_SYNCHRONIZE                                      0x00000001 /* Option indicating NO SYNCHRONIZE on connect       */

//-- XPM printer access error codes from GetLastError()
#define D_XPM_API_PIO_ERR_OUTOFMEMORY                                         0x40000001 /* Error code indicating out of memory               */
#define D_XPM_API_PIO_ERR_STOPSPOOLERFAILED                                   0x40000002 /* Error code indicating stop spooler failed         */
#define D_XPM_API_PIO_ERR_INVALIDPARAMETER                                    0x40000003 /* Error code indicating invalid parameter           */
#define D_XPM_API_PIO_ERR_ENDOFENUM                                           0x40000004 /* Error code indicating end of enumeration          */
#define D_XPM_API_PIO_ERR_OPENPRINTERFAILED                                   0x40000005 /* Error code indicating open printer failed         */
#define D_XPM_API_PIO_ERR_READPRINTERFAILED                                   0x40000006 /* Error code indicating read printer failed         */
#define D_XPM_API_PIO_ERR_WRITEPRINTERFAILED                                  0x40000007 /* Error code indicating write printer failed        */
#define D_XPM_API_PIO_ERR_UNKNOWNPRINTER                                      0x40000008 /* Error code indicating unknown printer             */

//-- Maximum values
#define D_XPM_API_PIO_S_PRINTER_NAME                                          200        /* Maximum length of printer name (excl. zero)       */
#define D_XPM_API_PIO_S_COMMAND                                               2000       /* Maximum length of command (excl. zero)            */

//-- Protocol revision 
#define D_XPM_API_PCKT_REVISION                                               0

//-- Data packets id's which correspond with parameter ubId in TS_XPM_API_PCKT_HEADER
//.. Note that the packet header "ESC [FF]+[i]+[n]" has [i] as packet identifier and [n] as packet data size.
//.. The packet data "[d1]+[dn]" immediately follows the packet header.
#define D_XPM_API_PID_TX_PRINTER_STATUS                                       2
#define D_XPM_API_PID_TX_PRINTER_INFO                                         3
#define D_XPM_API_PID_TX_PRINTER_SENSOR                                       4
#define D_XPM_API_PID_TX_PRINTER_FLASH                                        5
#define D_XPM_API_PID_TX_PRINTER_FEED                                         6
#define D_XPM_API_PID_TX_PRINTER_BARCODE                                      7
#define D_XPM_API_PID_TX_PRINTER_PRESENT                                      8
#define D_XPM_API_PID_TX_PRINTER_LIFE                                         9 
#define D_XPM_API_PID_TX_PRINTER_TICKET                                       10  // Ticket monitoring information

//-- Data request id's which correspond with bits in parameter [n] in emulation command "GS A+[n1]+[n2]"
//.. After reception of this command, the printer will send a sequence of requested data packets
//.. The packet data "[d1]+[dn]" immediately follows the packet header.
#define D_XPM_API_RID_RX_PRINTER_STATUS                                       0x0001
#define D_XPM_API_RID_RX_PRINTER_INFO                                         0x0002
#define D_XPM_API_RID_RX_PRINTER_SENSORS                                      0x0004
#define D_XPM_API_RID_RX_PRINTER_FLASH                                        0x0008
#define D_XPM_API_RID_RX_PRINTER_FEED                                         0x0010
#define D_XPM_API_RID_RX_PRINTER_BARCODE                                      0x0020
#define D_XPM_API_RID_RX_PRINTER_PRESENT                                      0x0040    
#define D_XPM_API_RID_RX_PRINTER_LIFE                                         0x0080    
#define D_XPM_API_RID_RX_PRINTER_BARCODES                                     0x0100 // Multi barcodes
#define D_XPM_API_RID_RX_PRINTER_TICKET                                       0x0200 // Ticket monitoring information

//-- Printer status packet -> printer status dword bits
#define D_XPM_API_PSDW_PRINTER_ERROR                                          0x00000001
#define D_XPM_API_PSDW_PRINT_HEAD_TEMP_ALERT                                  0x00000002
#define D_XPM_API_PSDW_FLASH_CHECKSUM_ERROR                                   0x00000004
#define D_XPM_API_PSDW_PRINTER_STALLED                                        0x00000008
#define D_XPM_API_PSDW_PAPER_OUT_DETECTED                                     0x00000010
#define D_XPM_API_PSDW_PAPER_LOW_A_DETECTED                                   0x00000020
#define D_XPM_API_PSDW_PAPER_MARK_DETECTED                                    0x00000040
#define D_XPM_API_PSDW_PAPER_IN_CHUTE_DETECTED                                0x00000080
#define D_XPM_API_PSDW_PRINT_HEAD_UP_DETECTED                                 0x00000100
#define D_XPM_API_PSDW_LAST_PRINTOUT_LOST                                     0x00000200
#define D_XPM_API_PSDW_AUXILIARY_SENSOR_ACTIVE                                0x00000400
#define D_XPM_API_PSDW_SYSTEM_FAILURE                                         0x00000800
#define D_XPM_API_PSDW_POWER_FAILURE                                          0x00001000
#define D_XPM_API_PSDW_CUSTOMER_FLAG_1                                        0x00002000
#define D_XPM_API_PSDW_CUSTOMER_FLAG_2                                        0x00004000
#define D_XPM_API_PSDW_PRINTER_NOT_READY                                      0x00008000
#define D_XPM_API_PSDW_PAPER_LOW_B_DETECTED                                   0x00010000
#define D_XPM_API_PSDW_BARCODE_SCANNED                                        0x00020000
#define D_XPM_API_PSDW_PRINT_UNIT_EMTPY                                       0x00040000

//-- Printer status packet -> Printer error word codes, low byte is WHAT, high byte is WHEN
#define D_XPM_API_PECW_WHAT_PRINTER_JAMMED                                    1  
#define D_XPM_API_PECW_WHAT_CUTTER_JAMMED                                     2  
#define D_XPM_API_PECW_WHAT_FEEDER_JAMMED                                     3
#define D_XPM_API_PECW_WHAT_PRESENTER_JAMMED                                  4
#define D_XPM_API_PECW_WHEN_WHILE_PRINTING                                    1
#define D_XPM_API_PECW_WHEN_WHILE_LOADING_PAPER                               2
#define D_XPM_API_PECW_WHEN_WHILE_ENDING_PAPER                                3
#define D_XPM_API_PECW_WHEN_WHILE_INITIALIZING                                4
#define D_XPM_API_PECW_WHEN_WHILE_ENDING_PRINT                                5
#define D_XPM_API_PECW_WHEN_WHILE_IDLE                                        6
#define D_XPM_API_PECW_WHEN_WHILE_RETRACTING_PAPER                            7
#define D_XPM_API_PECW_WHEN_WHILE_ACTIVATING_PAPER                            8
#define D_XPM_API_PECW_WHEN_WHILE_PARKING_PAPER                               9
#define D_XPM_API_PECW_WHEN_WHILE_EMPTY                                       10
#define D_XPM_API_PECW_WHEN_WHILE_EJECTING_PRINT                              11
#define D_XPM_API_PECW_WHEN_WHILE_INVALIDATING_PAPER                          12
#define D_XPM_API_PECW_WHEN_WHILE_LOADING_PRINT                               13
#define D_XPM_API_PECW_WHEN_WHILE_PRESENTING_PRINT                            14
#define D_XPM_API_PECW_WHEN_WHILE_REJECTING_PRINT                             15
#define D_XPM_API_PECW_WHEN_WHILE_RETRACTING_PRINT                            16
#define D_XPM_API_PECW_WHEN_WHILE_CANCELLING_PRINT                            17
#define D_XPM_API_PECW_WHEN_WHILE_UNJAMMING                                   18
#define D_XPM_API_PECW_WHAT_ERROR(ERR,WHAT)                                   (((ERR)&0xFF00) == (((WHAT) & 0x00FF) << 8))
#define D_XPM_API_PECW_WHEN_ERROR(ERR,WHEN)                                   (((ERR)&0x00FF) == ((WHEN) & 0x00FF))

//-- Printer status packet -> Printer error word information, enumeration 
#define D_XPM_API_PEIW_NONE                                                                                                   0
//.. Print unit
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_INVALID_PAPER_CODE_IN_EEPROM_STORAGE                                        1000
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_OUSIDE_PARK_AREA_WHEREAS_UNJAMMING_ON_INIT_IS_DISABLED                1001
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_COULD_NOT_RETRACT_PAPER_INTO_PRINT_UNIT                                     1002
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_DETECTING_TOP_OF_PAPER           1003
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINS_OCCUPIED_WHILE_DETECTING_TOP_OF_PAPER             1004
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_ENTRY_SENSOR_WAS_UNOCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER              1005
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINED_OCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER            1006
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINED_UNOCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER          1007
#define D_XPM_API_PEIW_PRINTER_x_IDLE_x_CUTTER_JAMMED_WHILE_BEING_IDLE                                                        1008
#define D_XPM_API_PEIW_PRINTER_x_PRINTING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_PRINT                                       1009
#define D_XPM_API_PEIW_PRINTER_x_ACTIVATING_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED_WHILE_ACTIVATING_PAPER                          1010
#define D_XPM_API_PEIW_PRINTER_x_ACTIVATING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_ACTIVATING_PAPER                           1011
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_WAS_OCCUPIED_WHILE_LOADING_PAPER                                       1012
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_ENTRY_SENSOR_WAS_UNOCCUPIED_WHILE_LOADING_PAPER                                    1013
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_WAS_OCCUPIED_TOO_EARLY_WHILE_LOADING_PAPER                             1014
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_LOADING_PAPER                                 1015
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_CUTTER_JAMMED_DURING_TEST                                                          1016
#define D_XPM_API_PEIW_PRINTER_x_ENDING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                         1017
#define D_XPM_API_PEIW_PRINTER_x_CANCELLING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                     1018
#define D_XPM_API_PEIW_PRINTER_x_CANCELLING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_EJECT                                     1019
#define D_XPM_API_PEIW_PRINTER_x_RETRACTING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                     1020
#define D_XPM_API_PEIW_PRINTER_x_EJECTING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                       1021
//.. Cutter unit
#define D_XPM_API_PEIW_CUTTER_x_INITIALIZATION_x_CUTTER_JAMMED_DURING_INITIALIZATION                                          2000
#define D_XPM_API_PEIW_CUTTER_x_INITIALIZATION_x_CUTTER_JAMMED_DURING_TEST                                                    2001
#define D_XPM_API_PEIW_CUTTER_x_LOADING_x_CUTTER_JAMMED_DURING_PAPER_LOAD                                                     2002
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_AT_END_OF_FORM                                                        2003
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_AT_END_OF_PRINT                                                       2004
#define D_XPM_API_PEIW_CUTTER_x_PARKED_x_CUTTER_JAMMED_WHILE_BEING_IDLE                                                       2005 
#define D_XPM_API_PEIW_CUTTER_x_UNJAMMING_x_CUTTER_JAMMED_DURING_UNJAMMING                                                    2006
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_DURING_TEST                                                           2007
//.. Feed unit, dual
#define D_XPM_API_PEIW_FEEDER_DUAL_x_LOADING_x_PRINT_UNIT_REFUSED_TO_LOAD_PAPER                                               3000
#define D_XPM_API_PEIW_FEEDER_DUAL_x_LOADING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                          3001
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                       3002
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_ACTIVATION                  3003
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_PRINT_UNIT_ENTRY_SENSOR_REMAINS_UNOCCUPIED_DURING_ACTIVATION                3004
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_PRINT_UNIT_REFUSED_TO_ACTIVATE_PAPER                                        3005
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_PARKING                        3006
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_PRINTER_UNIT_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_PARKING                      3007
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_PARKING                          3008
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_PRINT_UNIT_REFUSED_TO_RETRACT_PAPER                                            3009
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_FAILED_TO_RETRACT_PAPER_FROM_THE_PRINT_UNIT                          3010
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_PRINTER_UNIT_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                   3011
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                       3012
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_PRINT_UNIT_REFUSED_TO_RETRACT_PAPER                                         3013
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_FEED_UNIT_FAILED_TO_RETRACT_PAPER_FROM_THE_PRINT_UNIT                       3014
#define D_XPM_API_PEIW_FEEDER_DUAL_x_INITIALIZATION_x_PRINT_UNIT_HAS_MULTIPLE_PAPER_PATHS_COUPLED                             3015
//.. Presenter unit => METIS
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                     4000
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_BUNDLER_SENSOR_DID_NOT_DETECT_PAPER                                 4001
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_REJECT_SENSOR_DID_NOT_DETECT_PAPER                                  4002
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_REJECT_SENSOR_REMAINS_OCCUPIED                                      4003
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_BUNDLER_SENSOR_REMAINS_OCCUPIED                                     4004
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_BUNDLER_CAPACITY_EXHAUSTED                                                 4005
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_DECEEDS_MINIMUM                                               4006
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                          4007
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_EXCEEDS_MAXIMUM                                               4008
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_MISMATCHES_BUNDLED_PAGES                                      4009
#define D_XPM_API_PEIW_PRESENTER_METIS_x_PARKING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                   4010
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                  4011
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_UNOCCUPIED                                  4012
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                    4013
#define D_XPM_API_PEIW_PRESENTER_METIS_x_RETRACTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                  4014
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                 4015
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_REJECT_SENSOR_REMAINS_UNOCCUPIED                                  4016
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_REJECT_SENSOR_REMAINS_OCCUPIED                                    4017
//.. Presenter unit => BA 
#define D_XPM_API_PEIW_PRESENTER_BA_x_LOADING_PRINT_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED                                         4100
#define D_XPM_API_PEIW_PRESENTER_BA_x_LOADING_PRINT_x_EXIT_SENSOR_REMAINS_UNOCCUPIED                                          4101
#define D_XPM_API_PEIW_PRESENTER_BA_x_EJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED                                          4102
#define D_XPM_API_PEIW_PRESENTER_BA_x_EJECTING_PRINT_x_EXIT_SENSOR_REMAINS_OCCUPIED                                           4103
//.. Presenter unit => LOOP/RETRACT
#define D_XPM_API_PEIW_PRESENTER_LR_x_UNDEFINED_x_FLAP_SENSOR_REMAINS_OCCUPIED_AFTER_OPENING                                  4200
#define D_XPM_API_PEIW_PRESENTER_LR_x_UNDEFINED_x_FLAP_SENSOR_REMAINS_FREE_AFTER_CLOSING                                      4201
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_EXIT_SENSOR_REMAINS_OCCUPIED_AFTER_RETRACT                             4202
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                             4203
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                            4204
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED                                         4205
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_EXIT_SENSOR_REMAINS_UNOCCUPIED                                          4206
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_PAGE_LENGTH_DECEEDS_MINIMUM                                             4207
#define D_XPM_API_PEIW_PRESENTER_LR_x_EJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_EJECT                             4208
#define D_XPM_API_PEIW_PRESENTER_LR_x_RETRACTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_EJECT                           4209
#define D_XPM_API_PEIW_PRESENTER_LR_x_REJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                           4210
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_ENTRY_SENSOR_OCCUPIED_TOO_EARLY                                         4211

//-- Printer sensor packet -> printer sensor state 
#define D_XPM_API_PSS_MSN_NONE                                                0x00
#define D_XPM_API_PSS_MSN_REFLEX                                              0x10
#define D_XPM_API_PSS_MSN_TLIGHT                                              0x20
#define D_XPM_API_PSS_MSN_SWITCH                                              0x30
#define D_XPM_API_PSS_LSN_NONE                                                0x00
#define D_XPM_API_PSS_LSN_NO_PAPER                                            0x01
#define D_XPM_API_PSS_LSN_PAPER                                               0x02
#define D_XPM_API_PSS_LSN_PARKED                                              0x03
#define D_XPM_API_PSS_LSN_CUTTING                                             0x04
#define D_XPM_API_PSS_LSN_HEAD_DOWN                                           0x05
#define D_XPM_API_PSS_LSN_HEAD_UP                                             0x06
#define D_XPM_API_PSS_LSN_ACTIVE                                              0x07
#define D_XPM_API_PSS_LSN_INACTIVE                                            0x08
#define D_XPM_API_PSS_LSN_PRESSED                                             0x09
#define D_XPM_API_PSS_LSN_RELEASED                                            0x0A

//-- Printer flash packet -> printer flash area index/state (high/low nibble)
#define D_XPM_API_PFAI_FIRMWARE                                               0x00
#define D_XPM_API_PFAI_FONT_1                                                 0x01
#define D_XPM_API_PFAI_LOGO                                                   0x02
#define D_XPM_API_PFAI_FONT_2                                                 0x03
#define D_XPM_API_PFAI_WIDE_FONT                                              0x04
#define D_XPM_API_PFAI_IMAGES                                                 0x05
#define D_XPM_API_PFAS_UNDEFINED                                              0x00
#define D_XPM_API_PFAS_OK                                                     0x01
#define D_XPM_API_PFAS_ERROR_CRC                                              0x02
#define D_XPM_API_PFAS_ERROR_TYPE                                             0x03
#define D_XPM_API_PFAS_ERROR_SIZE                                             0x04

//-- Printer feed packet -> printer feed status dword bits
#define D_XPM_API_PFSW_ERROR                                                  0x0001
#define D_XPM_API_PFSW_ACTIVE                                                 0x0002
#define D_XPM_API_PFSW_PARKED                                                 0x0004
#define D_XPM_API_PFSW_PAPER_OUT_DETECTED                                     0x0008
#define D_XPM_API_PFSW_PAPER_LOW_A_DETECTED                                   0x0010
#define D_XPM_API_PFSW_PAPER_LOW_B_DETECTED                                   0x0020

//-- Presenter status packet -> printer present status word
#define D_XPM_API_PPSW_PRINTOUTS_LOADED                                       0x0001
#define D_XPM_API_PPSW_PRESENTING_PRINTOUTS                                   0x0002
#define D_XPM_API_PPSW_REJECTING_PRINTOUTS                                    0x0004
#define D_XPM_API_PPSW_PRINTOUTS_TAKEN                                        0x0008
#define D_XPM_API_PPSW_PRINTOUTS_REJECTED                                     0x0010
#define D_XPM_API_PPSW_PRESENTER_JAMMED                                       0x0020

//-- Printer ticket packet -> ticket status word
#define D_XPM_API_PTSW_BUSY                                                   0x0001 // Ticket is still in process (printing, cutting, presenting, etc)
#define D_XPM_API_PTSW_COMPLETED                                              0x0002 // Ticket has been printed, cutted and presented and either taken or rejected successfully 
#define D_XPM_API_PTSW_FAILED                                                 0x0004 // Ticket generation has failed (see other flags to determine where)
#define D_XPM_API_PTSW_PRINTED                                                0x0008 // Ticket has been printed successfully 
#define D_XPM_API_PTSW_CUTTED                                                 0x0010 // Ticket has been cutted successfully 
#define D_XPM_API_PTSW_LOADED                                                 0x0020 // Ticket has been loaded into the presenter successfully
#define D_XPM_API_PTSW_PRESENTED                                              0x0040 // Ticket(s) have been presented to the user successfully
#define D_XPM_API_PTSW_EJECTED_OR_TAKEN                                       0x0080 // Ticket(s) have been ejected or taken by the user 
#define D_XPM_API_PTSW_REJECTED                                               0x0100 // Ticket(s) have been rejected into the paper bin 

//-- Configuration Data - UART Baudrates  
#define D_XPM_API_CD_RS232_BAUDRATE_4800                                      0
#define D_XPM_API_CD_RS232_BAUDRATE_9600                                      1
#define D_XPM_API_CD_RS232_BAUDRATE_19200                                     2
#define D_XPM_API_CD_RS232_BAUDRATE_38400                                     3
#define D_XPM_API_CD_RS232_BAUDRATE_57600                                     4
#define D_XPM_API_CD_RS232_BAUDRATE_115200                                    5
#define D_XPM_API_CD_RS232_BAUDRATE_230400                                    6  /* Not supported */
#define D_XPM_API_CD_RS232_BAUDRATE_460800                                    7  /* Not supported */
#define D_XPM_API_CD_RS232_BAUDRATE_MAX                                       D_XPM_API_CD_RS232_BAUDRATE_115200
#define D_XPM_API_CD_RS232_BAUDRATE_DEF                                       D_XPM_API_CD_RS232_BAUDRATE_115200

//-- Configuration Data - UART parity on/off
#define D_XPM_API_CD_RS232_PARITY_OFF                                         0
#define D_XPM_API_CD_RS232_PARITY_ON                                          1

//-- Configuration Data - UART parity odd/even (if on)
#define D_XPM_API_CD_RS232_PARITY_ODD                                         0
#define D_XPM_API_CD_RS232_PARITY_EVEN                                        1

//-- Configuration Data - UART data bits (7/8)
#define D_XPM_API_CD_RS232_DATABIT_7                                          0  /* Not supported */
#define D_XPM_API_CD_RS232_DATABIT_8                                          1

//-- Configuration Data - UART stop bits (1/2)
#define D_XPM_API_CD_RS232_STOPBIT_1                                          0
#define D_XPM_API_CD_RS232_STOPBIT_2                                          1

//-- Configuration Data - UART flow control (handshake)
#define D_XPM_API_CD_RS232_FLOWCNTL_NONE                                      0  /* Not supported */
#define D_XPM_API_CD_RS232_FLOWCNTL_HW                                        1
#define D_XPM_API_CD_RS232_FLOWCNTL_SW                                        2  /* Not supported */
 
//-- Configuration Data - Wide font setting
#define D_XPM_API_CD_WF_NONE                                                  0 /* Wide font disabled */
#define D_XPM_API_CD_WF_STANDARD_24X24                                        1 /* 24x24 Standard wide font 0xE000..0xFFFF */
#define D_XPM_API_CD_WF_GB2312_CHINESE_24X24                                  2 /* 24x24 GB2312 Chinese wide font 0xA000..0xAFFFF */
#define D_XPM_API_CD_WF_STANDARD_20X20                                        3 /* 20x20 Standard wide font 0xE000..0xFFFF  */
#define D_XPM_API_CD_WF_KS6501_KOREAN_20X20                                   4 /* 20x20 KS5601 Korean font 0xA000..0xAFFFF */

//-- Configuration Data - Printer Options
#define D_XPM_API_CD_PRO_CUTTER_INIT_ON_POWER_ON                              0x00000001
#define D_XPM_API_CD_PRO_DISABLE_INQUIRE_TOP_OF_FOR_ON_INIT                   0x00000002
#define D_XPM_API_CD_PRO_DISABLE_UNJAM_PAPER_ON_INIT                          0x00000004
#define D_XPM_API_CD_PRO_INVALIDATE_PAPER_ON_INIT                             0x00000008
#define D_XPM_API_CD_PRO_INVALIDATE_PAPER_ON_DATA_LOST                        0x00000010
#define D_XPM_API_CD_PRO_SETUP_SINGLE_PATH_ONLY                               0x00000020
#define D_XPM_API_CD_PRO_ADD_CUSTOMER_FLAG_TO_USB_VERSION                     0x00000040 
#define D_XPM_API_CD_PRO_PRINT_WHOLE_OBJECTS_ONLY                             0x00000080 

//-- Configuration Data - Paper Path Options
#define D_XPM_API_CD_PPO_DISABLE_PRINT_INFO_ON_LOAD                           0x00000001
#define D_XPM_API_CD_PPO_DISABLE_CUT_ON_LOAD                                  0x00000002
#define D_XPM_API_CD_PPO_INVALIDATE_PAPER_ON_LOAD                             0x00000004
#define D_XPM_API_CD_PPO_DISABLE_RETRACT_ON_PAPER_END                         0x00000008
#define D_XPM_API_CD_PPO_INVALIDATE_PAPER_ON_PAPER_END                        0x00000010
#define D_XPM_API_CD_PPO_FORM_FEED_TO_MARK                                    0x00000020
#define D_XPM_API_CD_PPO_DISABLE_PAPER_LOAD_SENSOR_DETECT                     0x00000040
#define D_XPM_API_CD_PPO_DISABLE_PAPER_LOAD_BUTTON_CONTROL                    0x00000080
#define D_XPM_API_CD_PPO_REDUCE_PRINTOUT_TO_FORM_LENGTH                       0x00000100
#define D_XPM_API_CD_PPO_REDUCE_REVERSE_PAPER_MOVEMENT                        0x00000200
#define D_XPM_API_CD_PPO_WHOLE_INITIAL_FORM_ON_PAPER_LOAD                     0x00000400
#define D_XPM_API_CD_PPO_DISABLE_EJECT_ON_PAPER_END                           0x00000800
#define D_XPM_API_CD_PPO_PREPULL_FORM_BEFORE_PRINT                            0x00001000
#define D_XPM_API_CD_PPO_MARK_CUT_OFFSET_IS_RELATIVE_TO_MARK_CENTER           0x00002000

//-- Configuration Data - Printout Triggering Code
#define D_XPM_API_CD_PTC_ORIGINATOR_MASK                                      0xF0
#define D_XPM_API_CD_PTC_ORIGINATOR_NONE                                      0x00
#define D_XPM_API_CD_PTC_ORIGINATOR_AUXILIARY_SENSOR_HIGH                     0x10
#define D_XPM_API_CD_PTC_ORIGINATOR_AUXILIARY_SENSOR_LOW                      0x20
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_A1_HIGH              0x30
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_A1_LOW               0x40
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_A2_HIGH              0x50
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_A2_LOW               0x60
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_B1_HIGH              0x70
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_B1_LOW               0x80
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_B2_HIGH              0x90
#define D_XPM_API_CD_PTC_ORIGINATOR_PAPER_PRE_END_SENSOR_B2_LOW               0xA0
#define D_XPM_API_CD_PTC_PRINTOUT_MASK                                        0x0F
#define D_XPM_API_CD_PTC_PRINTOUT_NONE                                        0x00
#define D_XPM_API_CD_PTC_PRINTOUT_INFO_PAGE                                   0x01
#define D_XPM_API_CD_PTC_PRINTOUT_BLANK_PAGE                                  0x02
#define D_XPM_API_CD_PTC_PRINTOUT_SENSOR_READINGS                             0x03
#define D_XPM_API_CD_PTC_PRINTOUT_INVALIDATE                                  0x04
#define D_XPM_API_CD_PTC_PRINTOUT_MIXED_PATTERN                               0x05
#define D_XPM_API_CD_PTC_PRINTOUT_DIAGONAL_1                                  0x06
#define D_XPM_API_CD_PTC_PRINTOUT_DIAGONAL_2                                  0x07
#define D_XPM_API_CD_PTC_PRINTOUT_GRID_1                                      0x08
#define D_XPM_API_CD_PTC_PRINTOUT_GRID_2                                      0x09
#define D_XPM_API_CD_PTC_PRINTOUT_LINES_1                                     0x0A
#define D_XPM_API_CD_PTC_PRINTOUT_LINES_2                                     0x0B
#define D_XPM_API_CD_PTC_PRINTOUT_RHOMBUS_1                                   0x0C
#define D_XPM_API_CD_PTC_PRINTOUT_RHOMBUS_2                                   0x0D

//-- Configuration Data - Test page code 
#define D_XPM_API_CD_TPC_TEST_PAGE_1_MASK                                     0x0F
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DISABLED                                 0x00
#define D_XPM_API_CD_TPC_TEST_PAGE_1_BLANK_PAGE                               0x01
#define D_XPM_API_CD_TPC_TEST_PAGE_1_SENSOR_READINGS                          0x02
#define D_XPM_API_CD_TPC_TEST_PAGE_1_INVALIDATE                               0x03
#define D_XPM_API_CD_TPC_TEST_PAGE_1_MIXED_PATTERN                            0x04
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DIAGONAL_1                               0x05
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DIAGONAL_2                               0x06
#define D_XPM_API_CD_TPC_TEST_PAGE_1_GRID_1                                   0x07
#define D_XPM_API_CD_TPC_TEST_PAGE_1_GRID_2                                   0x08
#define D_XPM_API_CD_TPC_TEST_PAGE_1_LINES_1                                  0x09
#define D_XPM_API_CD_TPC_TEST_PAGE_1_LINES_2                                  0x0A
#define D_XPM_API_CD_TPC_TEST_PAGE_1_RHOMBUS_1                                0x0B
#define D_XPM_API_CD_TPC_TEST_PAGE_1_RHOMBUS_2                                0x0C
#define D_XPM_API_CD_TPC_TEST_PAGE_2_MASK                                     0xF0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DISABLED                                 0x00
#define D_XPM_API_CD_TPC_TEST_PAGE_2_BLANK_PAGE                               0x10
#define D_XPM_API_CD_TPC_TEST_PAGE_2_SENSOR_READINGS                          0x20
#define D_XPM_API_CD_TPC_TEST_PAGE_2_INVALIDATE                               0x30
#define D_XPM_API_CD_TPC_TEST_PAGE_2_MIXED_PATTERN                            0x40
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DIAGONAL_1                               0x50
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DIAGONAL_2                               0x60
#define D_XPM_API_CD_TPC_TEST_PAGE_2_GRID_1                                   0x70
#define D_XPM_API_CD_TPC_TEST_PAGE_2_GRID_2                                   0x80
#define D_XPM_API_CD_TPC_TEST_PAGE_2_LINES_1                                  0x90
#define D_XPM_API_CD_TPC_TEST_PAGE_2_LINES_2                                  0xA0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_RHOMBUS_1                                0xB0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_RHOMBUS_2                                0xC0

//-- External LED control  
#define D_XPM_API_CD_EXTLED_DISABLED                                          0x00000000
#define D_XPM_API_CD_EXTLED_ONBOARD_LEDS                                      0x00000001
#define D_XPM_API_CD_EXTLED_STATE_INDICATOR                                   0x00000002

//-- Configuration data - customer sensors - paper pre end sensor A #1 - enumeration
#define D_XPM_API_CD_CS_PPEA1_MASK                                            0x00000003
#define D_XPM_API_CD_CS_PPEA1_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA1_REFLEX                                          0x00000001
#define D_XPM_API_CD_CS_PPEA1_TLIGHT                                          0x00000002
#define D_XPM_API_CD_CS_PPEA1_SWITCH                                          0x00000003
#define D_XPM_API_CD_CS_PPEA1_ACTIVE_WHEN_LOW                                 0x00000008
#define D_XPM_API_CD_CS_PPEA1_PASS_THROUGH                                    0x00000004

//-- Configuration data - customer sensors - paper pre end sensor A #2 - enumeration
#define D_XPM_API_CD_CS_PPEA2_MASK                                            0x00000030
#define D_XPM_API_CD_CS_PPEA2_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA2_REFLEX                                          0x00000010
#define D_XPM_API_CD_CS_PPEA2_TLIGHT                                          0x00000020
#define D_XPM_API_CD_CS_PPEA2_SWITCH                                          0x00000030
#define D_XPM_API_CD_CS_PPEA2_ACTIVE_WHEN_LOW                                 0x00000080
#define D_XPM_API_CD_CS_PPEA2_PASS_THROUGH                                    0x00000040

//-- Configuration data - customer sensors - paper pre end sensor A #3 - enumeration
#define D_XPM_API_CD_CS_PPEA3_MASK                                            0x00000300
#define D_XPM_API_CD_CS_PPEA3_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA3_REFLEX                                          0x00000100
#define D_XPM_API_CD_CS_PPEA3_TLIGHT                                          0x00000200
#define D_XPM_API_CD_CS_PPEA3_SWITCH                                          0x00000300
#define D_XPM_API_CD_CS_PPEA3_ACTIVE_WHEN_LOW                                 0x00000800
#define D_XPM_API_CD_CS_PPEA3_PASS_THROUGH                                    0x00000400

//-- Configuration data - customer sensors - paper pre end sensor B #1 - enumeration
#define D_XPM_API_CD_CS_PPEB1_MASK                                            0x00003000
#define D_XPM_API_CD_CS_PPEB1_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB1_SWITCH                                          0x00001000
#define D_XPM_API_CD_CS_PPEB1_ACTIVE_WHEN_LOW                                 0x00008000
#define D_XPM_API_CD_CS_PPEB1_PASS_THROUGH                                    0x00004000

//-- Configuration data - customer sensors - paper pre end sensor B #2 - enumeration
#define D_XPM_API_CD_CS_PPEB2_MASK                                            0x00030000
#define D_XPM_API_CD_CS_PPEB2_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB2_SWITCH                                          0x00010000
#define D_XPM_API_CD_CS_PPEB2_ACTIVE_WHEN_LOW                                 0x00080000
#define D_XPM_API_CD_CS_PPEB2_PASS_THROUGH                                    0x00040000

//-- Configuration data - customer sensors - paper pre end sensor B #3 - enumeration
#define D_XPM_API_CD_CS_PPEB3_MASK                                            0x00300000
#define D_XPM_API_CD_CS_PPEB3_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB3_SWITCH                                          0x00100000
#define D_XPM_API_CD_CS_PPEB3_ACTIVE_WHEN_LOW                                 0x00800000
#define D_XPM_API_CD_CS_PPEB3_PASS_THROUGH                                    0x00400000

//-- Configuration data - customer sensors - auxiliary sensor - enumeration
#define D_XPM_API_CD_CS_AUX_MASK                                              0x07000000
#define D_XPM_API_CD_CS_AUX_NONE                                              0x00000000
#define D_XPM_API_CD_CS_AUX_REFLEX                                            0x01000000
#define D_XPM_API_CD_CS_AUX_TLIGHT                                            0x02000000
#define D_XPM_API_CD_CS_AUX_SWITCH                                            0x03000000
#define D_XPM_API_CD_CS_AUX_ACTIVE_WHEN_LOW                                   0x08000000

//-- Configuration data - customer sensors - reserved
#define D_XPM_API_CD_CS_RES_MASK                                              0xFFFF0000

//-- Number of Paper Paths
#define D_XPM_API_NR_OF_PAPER_PATHS                                           3 

/*
** TYPES
*/

//-- Printer packet header
#pragma pack(1)
typedef struct   
   { BYTE         ubSync1;                                  // First synchronization byte (must be 0x1B)
     BYTE         ubSync2;                                  // Second synchronization byte (must be 0xFF)
     BYTE         ubId;                                     // Identification of packet (see D_XPM_API_PID_TX_....)
     BYTE         ubLength;                                 // Length of header and data part
   } TS_XPM_API_PCKT_HEADER;                                // Packet header
#pragma pack()

//-- Printer status packet
#pragma pack(1)
typedef struct   
   { DWORD        dwStatus;                                 // Printer status dword as in D_XPM_API_PSDW_...
     BYTE         ubPrintJobId;                             // Id of print job
     SHORT        siPrintHeadTempC;                         // Printhead temperature in degrees celcius (ADC#02)
     WORD         wBoardVoltageTv;                          // Board voltage in 1/10 volt
     BYTE         ubState;                                  // State of printer control
     WORD         wErrorCode;                               // Printer error code word as in D_XPM_API_PECW_...
     WORD         wErrorInfo;                               // Printer error info word as in D_XPM_API_PEIW_...
   } TS_XPM_API_PCKT_PRINTER_STATUS;                        // Printer status packet
#pragma pack()

//-- Printer info packet
#pragma pack(1)
typedef struct 
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol (see D_XPM_API_PCKT_REVISION)
     CHAR         szPartNr[16+1];                           // Part number, inluding 0 terminator
     CHAR         szSerialNr[16+1];                         // Serial number, inluding 0 terminator
     CHAR         szDateOfManuf[16+1];                      // Date of manufacturing, inluding 0 terminator
     DWORD        dwModelCode;                              // Model code
     DWORD        dwConfigCode;                             // Configuration code
     DWORD        dwSensorCode;                             // Sensor code
     DWORD        dwFeatureCode;                            // Feature code
     CHAR         szFwPartNr[16+1];                         // Part number, inluding 0 terminator 
     CHAR         szFwVersion[10+1];                        // Firmware version  Rx-Vy.zzb
     CHAR         szFwDate[12+1];                           // Firmware date     Mmm dd yyyy (ANSI standard).
     DWORD        dwFwPatch;                                // Firmware patch code
     BYTE         ubReserved[4];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_INFO;                          // Printer info packet
#pragma pack()

//-- Printer sensor packet
#pragma pack(1)
typedef struct
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     BYTE         ubAdcMeasurement[16];                     // ADC channel #00..#15 measurements in domain 0=0V..255=5V     (ADC#XX)
     WORD         wDigitalInput;                            // DIN digital inputs bits #00..#15                             (DIN#XX)
     SHORT        siPrintHeadTempC;                         // Printhead temperature in degrees celcius                     (ADC#02)
     WORD         wBoardVoltageTv;                          // Board voltage in tenths of volt                              (ADC#01)
     BYTE         ubPaperPreEndSensorAState[3];             // State byte for paper pre end sensor A     (D_XPM_API_PSS_..) (ADC#03/06/--)
     BYTE         ubPaperPreEndSensorBState[3];             // State byte for paper pre end sensor B     (D_XPM_API_PSS_..) (DIN#06/07/--)
     BYTE         ubPaperPathKeyState[3];                   // State byte for paper path keys            (D_XPM_API_PSS_..) (DIN#04/05/--)
     BYTE         ubPaperPathEntrySensorState[3];           // State byte for paper path entry sensors   (D_XPM_API_PSS_..) (ADC#08/09/--)
     BYTE         ubPaperPathExitSensorState[3];            // State byte for paper path exit sensors    (D_XPM_API_PSS_..) (ADC#10/11/--)
     BYTE         ubPaperEntrySensor1State;                 // State byte for paper entry sensor #1      (D_XPM_API_PSS_..) (ADC#04)
     BYTE         ubPaperEntrySensor2State;                 // State byte for paper entry sensor #2      (D_XPM_API_PSS_..) (ADC#05)
     BYTE         ubHeadUpSensorState;                      // State byte for head up sensor             (D_XPM_API_PSS_..) (DIN#00)
     BYTE         ubCutterSensor1State;                     // State byte for cutter sensor #1 (left)    (D_XPM_API_PSS_..) (DIN#01)
     BYTE         ubCutterSensor2State;                     // State byte for cutter sensor #2 (right)   (D_XPM_API_PSS_..) (DIN#02)
     BYTE         ubPaperExitSensorState;                   // State byte for paper exit sensor          (D_XPM_API_PSS_..) (ADC#00)
     BYTE         ubAuxiliarySensorState;                   // State byte for auxiliary sensor           (D_XPM_API_PSS_..) (ADC#12)
     BYTE         ubPresenterSensor1State;                  // State byte for presenter sensor #1        (D_XPM_API_PSS_..) (ADC#13)
     BYTE         ubPresenterSensor2State;                  // State byte for presenter sensor #2        (D_XPM_API_PSS_..) (ADC#14)
     BYTE         ubPresenterSensor3State;                  // State byte for presenter sensor #3        (D_XPM_API_PSS_..) (ADC#15)
     BYTE         ubReserved[5];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_SENSOR;                        // Printer sensor packet
#pragma pack()

//-- Printer flash packet
#pragma pack(1)
typedef struct 
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     CHAR         szInfo[6][20+1];                          // Flash area info, indexed by D_XPM_API_PFAI_..., inluding 0 terminator
     WORD         wCrc16[6][2];                             // Flash area 16 bit checksum, indexed by D_XPM_API_PFAI_..., 0=stored and 1=calculated
     BYTE         ubState[6];                               // Flash area state, indexed by D_XPM_API_PFAI_..., contets responds to D_XPM_API_PFAS_..., 
     BYTE         ubReserved[6];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_FLASH;                         // Printer flash packet
#pragma pack()

//-- Printer feed packet
#pragma pack(1)
typedef struct 
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     BYTE         ubState;                                  // State of the feed unit control 
     BYTE         ubNrOfPaperPaths;                         // Total number of paper path
     WORD         wPaperPathStatus[3];                      // Status word of each paper path as in D_XPM_API_PFSW_...
     BYTE         ubPaperPathState[3];                      // State of the paper path control for each paper path 
     BYTE         ubReserved[8];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_FEED;                          // Printer feed packet
#pragma pack()

//-- Printer present packet 
#pragma pack(1)
typedef struct  
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     BYTE         ubType;                                   // Type of presenter (0 = none, 1 = loop)
     WORD         wStatus;                                  // Presenter status word bits as in D_XPM_API_PPSW_...
     BYTE         ubState;                                  // State of presenter control (idle/loading/presenting/rejecting, etc.)
     BYTE         ubNrOfLoadedPrintouts;                    // Total number of currently loaded  (bundled) printouts
     DWORD        dwNrOfLoadedPrintouts;                    // Total number of loaded    printouts since start / power on
     DWORD        dwNrOfTakenPrintouts;                     // Total number of taken     printouts since start / power on
     DWORD        dwNrOfRejectedPrintouts;                  // Total number of rejected  printouts since start / power on
     BYTE         ubReserved[3];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_PRESENT;                       // Printer present packet
#pragma pack()

//-- Printer life cycle data packet
#pragma pack(1)
typedef struct 
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     DWORD        dwTimePoweredS;                           // Total time that the printer is powered in sec
     DWORD        dwDistanceSteppedMm;                      // Total distance stepped in mm
     DWORD        dwDistanceBurnedMm;                       // Total distance burned in mm
     DWORD        dwPaperTransportedMm;                     // Total length of transported paper in mm
     DWORD        dwNrOfSystemStarts;                       // Total number of system starts
     DWORD        dwNrOfHardResets;                         // Total number of hardware resets 
     DWORD        dwNrOfPaperCuts;                          // Total number of paper cuts 
     WORD         wNrOfPaperLoads;                          // Total number of paper loads 
     BYTE         ubReserved[8];                            // Reserved bytes (2 DWORDs, zero initialized)
   } TS_XPM_API_PCKT_PRINTER_LIFE;                          // Printer life cycle data packet
#pragma pack()

//-- Printer ticket packet 
#pragma pack(1)
typedef struct 
   { BYTE         ubProtocolRevision;                       // Revision of communication protocol
     BYTE         ubTicketId;                               // Ticket identifier [0..15] (automatic increase if undefined at start of ticket)
     WORD         wTicketLengthMm;                          // Total length of ticket as defined at start of print in millimetres (-1 if undefined at start of ticket)
     WORD         wPrintedLengthMm;                         // Distance of ticket printed (reset at start of print) (-1 if printed length is undefined after boot) 
     BYTE         ubPrintedAreaMinPrcnt;                    // Minimum percentage of ticket completed (-1 if ticket length is undefined at start of ticket) 
     BYTE         ubPrintedAreaMaxPrcnt;                    // Maximum percentage of ticket completed (-1 if ticket length is undefined at start of ticket) 
     WORD         wTicketStatus;                            // Status word of last printed ticket as in define D_XPM_API_PTSW_...
     BYTE         ubReserved[4];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_TICKET;                        // Printer ticket packet
#pragma pack()

//-- XPM configuration data - customer modificable
#pragma pack(1)
typedef struct 
   { WORD         wRevisionNumber;                          // Structure revision number 
     WORD         wSizeOfBlock;                             // Size of this structure
     CHAR         szCustDescription[16+1];                  // Customer-description, zero is unused
     BYTE         ubReserved1[17];                          // Reserved bytes for future use (must be 0)
     BYTE         ubCustFlags;                              // Customer specific flags
     BYTE         ubBaudRate;                               // Baudrate matching one of XPM_API_CD_RS232_BAUDRATE_...
     BYTE         ubParityOnOff;                            // Parity matching one of XPM_API_CD_RS232_PARITY_... (on/off)
     BYTE         ubParity;                                 // Parity matching one of XPM_API_CD_RS232_PARITY_... (even/odd, if on)
     BYTE         ubDataBits;                               // Data bits matching one of XPM_API_CD_RS232_DATABIT_... (7/8)
     BYTE         ubStopBits;                               // Stop bits matching one of XPM_API_CD_RS232_STOPBIT_... (1/2)
     BYTE         ubFlowControl;                            // Flow control matching one of XPM_API_CD_RS232_FLOWCNTL_... (SW/HW/NO)
     BYTE         ubBlockHostTxOnPaperOut;                  // Flag indicating disable host TX on paper out (RS232-HW/SW flow control only)
     BYTE         ubWideFont;                               // Enumeration indicating wide font set enabled if non zero
     DWORD        dwPrinterOptions;                         // Customer Printer options
     DWORD        dwCustSensors;                            // Customer auxiliary and paper pre end sensors as in XPM_API_CD_CS_....  
     WORD         wPrintSpeedTmms[3];                       // Print speed (default) in 1/10 mm/sec (for 3 paper paths)
     WORD         wPaperLoadSpeedTmms[3];                   // Paper load speed in 1/10 mm/s (for 3 paper paths)
     WORD         wPaperControlSpeedTmms[3];                // Paper control speed in 1/10 mm/s (for 3 paper paths)
     CHAR         cPrintDensity[3];                         // Print density (default) in percent (for 3 paper paths)
     BYTE         ubDhisBurnFac[3];                         // Dot history burn factor in percent of total burn time (zero indicates OFF) (for 3 paper paths)
     BYTE         ubMultiStrobeFac[3];                      // Multi strobe counter, zero is undefined (for 3 paper paths)
     CHAR         asbBurnTimeCorr[3][11];                   // Burn time corrections in us for -30..+70C with 10C intervals (obsolete) (for 3 paper paths)l
     DWORD        dwPaperPathOptions[3];                    // Customer Paper Path options (for 3 paper paths)
     DWORD        dwFormLengthYm[3];                        // Maximum form length in micrometer (for 3 paper paths)
     WORD         wMarkLengthYm[3];                         // Length of a black/hole mark in micrometer (for 3 paper paths)
     DWORD        dwMarkCutDisYm[3];                        // Distance from end of black/hole mark to cut position in micrometer (for 3 paper paths)
     WORD         wPaperEndDetectDisYm[3];                  // Minimum distance required before detecting paper end (for 3 paper paths)
     BYTE         ubPaperParkPos[3];                        // Paper park position 0 or 1 (for 3 paper paths)
     SHORT        iPresentOffsetYm[3];                      // Presentation offset in micrometer (for 3 paper paths)
     DWORD        dwPaperLoadFeedYm[3];                     // Feed length after paper load in micrometer (for 3 paper paths)
     BYTE         ubPaperLoadDurationS[3];                  // Maximum duration of paper load in seconds (for 3 paper paths)
     WORD         wPaperWeightGm2[3];                       // Paper weight in gram per square meter (for 3 paper paths)
     WORD         wPaperOffsetTmm[3];                       // Paper offset in 1/10 mm  (for 3 paper paths)
     WORD         wPaperWidthTmm[3];                        // Paper width in 1/10 mm  (0 is default)
     BYTE         ubPrintDataLostTimeoutS;                  // Timeout in sec for print data to be regarded lost 
     BYTE         ubExternalLedControl;                     // External LED control as in XPM_API_CD_EXTLED_... 
     WORD         wPresentSpeedTmms[3];                     // Present speed in 1/10 mm/s (only for particular presenters) (for 3 paper paths)
     WORD         wBundlerSpeedTmms[3];                     // Bundler speed in 1/10 mm/s (only for particular presenters) (for 3 paper paths)
     SHORT        iMarkCutCorrectionYm[3];                  // Mark cut correction in micrometer (for 3 paper paths)
     SHORT        iPaperBundleCorrectionYm[3];              // Paper bundle correction in micrometer (only for particular presenters) (for 3 paper paths)
     BYTE         ubTestPageCodeOnLoad[3];                  // Code for printout of test page #1 and #2 on paper load (for 3 paper paths)
     SHORT        iPrintCorrectionYm[3];                    // Correction of top of paper at start if print (unparking) in micrometer (for 3 paper paths)
     BYTE         ubNrOfCutRetries;                         // Total number of retries after a cut failure has been detected
     BYTE         ubTestPageCodeOnClose[3];                 // Code for printout of test page #1 and #2 on printer close (for 3 paper paths)
     BYTE         ubDhisLevel[3];                           // Total numbr of dot history level (0 = dot history disabled) (for 3 paper paths)
     BYTE         ubInitializationDelayTs;                  // Initialization delay in 1/10 seconds
     CHAR         acMarkThresholdCorrectionTv[3];           // Mark threshold correction in 1/10 volt (available since firmware V1.04i)
     BYTE         aubPrintoutTriggeringCode;                // Code for printout triggering (as in D_XPM_API_CD_PTC_...)
     BYTE         aubReserved2[1];                          // Reserved bytes for future use (complete structure is 0xFF bytes) (must be 0)
     WORD         wCrc16;                                   // 16 Bit checksum of structure
   } TS_XPM_API_CONFIG_DATA;                                // XPM configuration data structure as uploaded to and stored in flash
#define D_XPM_API_CONFIG_DATA_REV 1                         // Revision of configuration data
#pragma pack()

typedef enum
   { EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_1_FS             // Error diffusion dithering algorithm #1 (FS)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_2_ST             // Error diffusion dithering algorithm #2 (ST)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_3_AT             // Error diffusion dithering algorithm #3 (AT)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_4_JJN            // Error diffusion dithering algorithm #4 (JJN)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_5_S1             // Error diffusion dithering algorithm #5 (S1)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_6_S2             // Error diffusion dithering algorithm #6 (S2)
   , EV_XPM_API_GSC2MON_DA_ERROR_DIFFUSION_7_S3             // Error diffusion dithering algorithm #7 (S3)
   , EV_XPM_API_GSC2MON_DA_RANDOM_1_VALUE                   // Random dithering algorithm #1 which uses a random value generator for determining black or white
   , EV_XPM_API_GSC2MON_DA_ORDERED_1_FORM                   // Ordered dithering algorithm #1 which uses form values for determining black or white
   , EV_XPM_API_GSC2MON_DA_ORDERED_2_BY2X2                  // Ordered dithering algorithm #3 (B2X2)
   , EV_XPM_API_GSC2MON_DA_ORDERED_3_BY4X4                  // Ordered dithering algorithm #4 (B4X4)
   , EV_XPM_API_GSC2MON_DA_ORDERED_4_BY8X8                  // Ordered dithering algorithm #5 (B8X8)
   } ET_XPM_API_GSC2MON_DITHERING_ALGORITHM;                // All defined dithering algorithms for gray scale to monochrome conversion

typedef enum
   { EV_XPM_API_RGB2GSC_CA_AVERAGING                        // Averaging:     GRAY-SCALE = (R+G+B)/3
   , EV_XPM_API_RGB2GSC_CA_LUMINANCE                        // Luminance:     GRAY-SCALE = (R*2+G*5+B*1)/8
   , EV_XPM_API_RGB2GSC_CA_DESATURATION                     // Desaturation:  GRAY-SCALE = (MAX(R,G,B)+MIN(R,G,B))/2
   , EV_XPM_API_RGB2GSC_CA_DECOMPOSITION                    // Decomposition: GRAY-SCALE = MIN(R,G,B)
   } ET_XPM_API_RGB2GSC_CONVERSION_ALGORITHM;               // All defined conversion algorithms for colour to gray scale conversion 

/*
** FUNCTIONS
*/

//-- Printer access functions
HANDLE                    WINAPI Xpm_Api_Pio_PrinterOpen                  (LPSTR pszPrinterName, DWORD dwOptions, DWORD dwTimeOutMs);
bool                      WINAPI Xpm_Api_Pio_PrinterClose                 (HANDLE hPrinterIo);
DWORD                     WINAPI Xpm_Api_Pio_PrinterRead                  (HANDLE hPrinterIo, unsigned char * pubData, DWORD nData, DWORD dwTimeOutMs);
bool                      WINAPI Xpm_Api_Pio_PrinterReadClear             (HANDLE hPrinterIo);
DWORD                     WINAPI Xpm_Api_Pio_PrinterWrite                 (HANDLE hPrinterIo, unsigned char * pubData, DWORD nData, DWORD dwTimeOutMs);
bool                      WINAPI Xpm_Api_Pio_PrintImage                   (HANDLE hPrinterIo, LPSTR pszImageFileRgb, DWORD dwTimeOutMs);
bool                      WINAPI Xpm_Api_Pio_PrintImageEx                 (HANDLE hPrinterIo, LPSTR pszImageFileRgb, DWORD dwTimeOutMs
                                                                          ,ET_XPM_API_RGB2GSC_CONVERSION_ALGORITHM eRgb2GscConversionAlgorithm, int iRgb2GscRedAdjustment, int iRgb2GscGreenAdjustment, int iRgb2GscBlueAdjustment
                                                                          ,ET_XPM_API_GSC2MON_DITHERING_ALGORITHM eGsc2MonDitheringAlgorithm, WORD wGsc2MonBlackLevel, WORD wGsc2MonWhiteLevel, int iGsc2MonBrightness, int iGsc2MonContrast);

//-- Printer enumeration functions
HANDLE                    WINAPI Xpm_Api_Pe_PrinterEnumStart              (void);
bool                      WINAPI Xpm_Api_Pe_PrinterEnumEnd                (HANDLE hPrinterEnum);
LPSTR                     WINAPI Xpm_Api_Pe_PrinterEnum                   (HANDLE hPrinterEnum, bool fReset);
bool                      WINAPI Xpm_Api_Pe_ValidatePrinterName           (LPSTR lpszPrinter);

//-- Image conversion functions
bool                      WINAPI Xpm_Api_Pio_RenderImage                  (HANDLE hPrinterIo, LPSTR pszImageFileRgb, LPSTR pszImageFileGsc, LPSTR pszImageFileMon);
bool                      WINAPI Xpm_Api_Pio_RenderImageEx                (LPSTR pszImageFileRgb
                                                                          ,ET_XPM_API_RGB2GSC_CONVERSION_ALGORITHM eRgb2GscConversionAlgorithm, int iRgb2GscRedAdjustment, int iRgb2GscGreenAdjustment, int iRgb2GscBlueAdjustment
                                                                          ,ET_XPM_API_GSC2MON_DITHERING_ALGORITHM eGsc2MonDitheringAlgorithm, WORD wGsc2MonBlackLevel, WORD wGsc2MonWhiteLevel, int iGsc2MonBrightness, int iGsc2MonContrast
                                                                          ,LPSTR pszImageFileGsc, LPSTR pszImageFileMon);


/*
** VARIABLES
*/

#endif 
