/***********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmp_JobInfo.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Xpm-PrintMonPlugin)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the job info structure entry in the registry
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Xpm_Pmp_System.h"
#include "Xpm_PrintMonPlugin.h"
#include "Xpm_Pmp_Interface.h"
#include "Xpm_Pmp_RegistryAccess.h"
#include "Xpm_Pmp_JobInfo.h"

/*
** COMPILER 
*/

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

static VOID Lo_InitJobInfoStructure      (PS_XPM_PMP_JOB_INFO psJobInfo);

/*
** VARIABLES
*/

//******************************************************************************************
// FUNCTION   Lo_InitJobInfoStructure
// ABSTRACT   Initializes the job info structure 
//******************************************************************************************

static VOID Lo_InitJobInfoStructure
   (PS_XPM_PMP_JOB_INFO psJobInfo)
{
   //-- Initialize job info structure 
   memset(psJobInfo, 0, sizeof(*psJobInfo));             // Clear complete structure
   psJobInfo->dwSize          = sizeof(*psJobInfo);      // Set size of this structure
   psJobInfo->ubRevision      = D_XPM_PMP_JI_REVISION;  // Set revision number (D_XPM_PMP_JI_REVISION)
   psJobInfo->dwTsLastUpdate  = 0;                       // Set timestamp of last update
   psJobInfo->dwJobId         = (DWORD) -1;              // Set job identifier to undefined (0xfff...)
   psJobInfo->dwJobStatus     = 0;                       // Set job status to invalid
   psJobInfo->dwPageCount     = 0;                       // Set page count to zero
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Ji_InitializeJobInfo
// ABSTRACT   Initializes job information in registry
//******************************************************************************************

bool Xpm_Pmp_Ji_InitializeJobInfo
   (P_XPM_PLUGIN_DATA psData)
{
   bool                            fResult                    = false;                  // Return value, FALSE by default
   TS_XPM_PMP_JOB_INFO             sJobInfo;                                            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize job info structure 
   Lo_InitJobInfoStructure(&sJobInfo);

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Ji_UpdateJobInfo_JobStarted
// ABSTRACT   Updates job info structure in registry at start of print job 
//******************************************************************************************

bool Xpm_Pmp_Ji_UpdateJobInfo_JobStarted
   (P_XPM_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_XPM_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize job info structure 
   Lo_InitJobInfoStructure(&sJobInfo);

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as active
   sJobInfo.dwJobStatus = D_XPM_PMP_JI_JS_JOB_ACTIVE;  

   //-- Reset page number
   sJobInfo.dwPageCount = 0; 

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Ji_UpdateJobInfo_JobCompleted
// ABSTRACT   Updates job info structure in registry at end of print job 
//******************************************************************************************

bool Xpm_Pmp_Ji_UpdateJobInfo_JobCompleted
   (P_XPM_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_XPM_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Xpm_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_XPM_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as finished
   sJobInfo.dwJobStatus = D_XPM_PMP_JI_JS_JOB_FINISHED;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Ji_UpdateJobInfo_JobCancelled
// ABSTRACT   Updates job info structure in registry in case the print job is cancelled 
//******************************************************************************************

bool Xpm_Pmp_Ji_UpdateJobInfo_JobCancelled
   (P_XPM_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_XPM_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Xpm_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_XPM_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as finished
   sJobInfo.dwJobStatus = D_XPM_PMP_JI_JS_JOB_CANCELLED;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Ji_UpdateJobInfo_PageCompleted
// ABSTRACT   Updates job info structure in registry at end of page 
//******************************************************************************************

bool Xpm_Pmp_Ji_UpdateJobInfo_PageCompleted
   (P_XPM_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_XPM_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Xpm_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_XPM_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Increase page number, only if job identifier matches
   if (sJobInfo.dwJobId == dwJobId)
        sJobInfo.dwPageCount++; 
   else sJobInfo.dwPageCount = 0; 

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as active
   sJobInfo.dwJobStatus = D_XPM_PMP_JI_JS_JOB_ACTIVE;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
